@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright © 2018 Damir Jelić <poljar@termina.org.uk>
-# Copyright © 2020 Famedly GmbH
+# Copyright © 2020-2021 Famedly GmbH
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -370,7 +370,8 @@ class Api:
 
         Args:
             user (str): The fully qualified user ID or just local part of the
-                user ID, to log in.
+                user ID, to log in. If the user ID contains an '@', but no ':',
+                the user ID will be considered to be an email address.
             password (str): The user's password.
             device_name (str): A display name to assign to a newly-created
                 device. Ignored if device_id corresponds to a known device
@@ -381,9 +382,24 @@ class Api:
         path = Api._build_path(path=["login"])
 
         if password is not None:
+            identifier = {}
+            if '@' in user and not user.startswith('@'):
+                identifier = {
+                    'type': 'm.id.thirdparty',
+                    'medium': 'email',
+                    'address': user,
+                }
+            else:
+                # As per spec, a user can login with either their localpart (that
+                # cannot contain an @) or their full Matrix ID, starting with an @.
+                identifier = {
+                    'type': 'm.id.user',
+                    'user': user,
+                }
+
             content_dict = {
                 "type": "m.login.password",
-                "user": user,
+                "identifier": identifier,
                 "password": password,
             }
         elif token is not None:
@@ -815,6 +831,8 @@ class Api:
         invite=(),                          # type: Sequence[str]
         initial_state=(),                   # type: Sequence[Dict[str, Any]]
         power_level_override=None,          # type: Optional[Dict[str, Any]]
+        predecessor=None,                   # type: Optional[Dict[str, Any]]
+        space=False,                        # type: bool
     ):
         # type (...) -> Tuple[str, str, str]
         """Create a new room.
@@ -871,6 +889,8 @@ class Api:
                 to override the default.
                 The dict will be applied on top of the generated
                 ``m.room.power_levels`` event before it is sent to the room.
+
+            space (bool): Create as a Space (defaults to False).
         """
         path = ["createRoom"]
         query_parameters = {"access_token": access_token}
@@ -905,6 +925,12 @@ class Api:
         if power_level_override:
             body["power_level_content_override"] = power_level_override
 
+        if predecessor:
+            body["creation_content"]["predecessor"] = predecessor
+
+        if space:
+            body["creation_content"]["type"] = "m.space"
+
         return (
             "POST",
             Api._build_path(path, query_parameters),
@@ -1267,6 +1293,56 @@ class Api:
 
         return "GET", Api._build_path(path)
 
+    @staticmethod
+    def room_delete_alias(access_token, room_alias):
+        # type: (str, str) -> Tuple[str, str]
+        """Delete a room alias.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_alias (str): The alias to delete
+        """
+        query_parameters = {"access_token": access_token}
+        path = ["directory", "room", room_alias]
+
+        return "DELETE", Api._build_path(path, query_parameters)
+
+    @staticmethod
+    def room_put_alias(access_token, room_alias, room_id):
+        # type: (str, str, str) -> Tuple[str, str, str]
+        """Add a room alias.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_alias (str): The alias to add
+            room_id (str): The room ID to map to
+        """
+        query_parameters = {"access_token": access_token}
+        path = ["directory", "room", room_alias]
+        body = {
+            "room_id": room_id,
+        }
+
+        return "PUT", Api._build_path(path, query_parameters), Api.to_json(body)
+
+    @staticmethod
+    def room_get_visibility(room_id):
+        # type: (str) -> Tuple[str, str]
+        """Get visibility of a room in the directory.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            room_id (str): The room ID to query.
+        """
+        path = ["directory", "list", "room", room_id]
+
+        return "GET", Api._build_path(path)
+
     @staticmethod
     def room_typing(
         access_token,       # type: str
@@ -1914,3 +1990,46 @@ class Api:
             Api._build_path(path, query_parameters),
             Api.to_json(content),
         )
+
+    @staticmethod
+    def delete_room_alias(
+            access_token: str,
+            alias: str) -> Tuple[str, str]:
+        """Delete an room alias
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            alias (str): The room alias
+        """
+
+        query_parameters = {"access_token": access_token}
+        path = ["directory", "room", alias]
+
+        return ("DELETE",
+                Api._build_path(path, query_parameters))
+
+    @staticmethod
+    def put_room_alias(
+            access_token: str,
+            alias: str,
+            room_id: str) -> Tuple[str, str, str]:
+        """Add an room alias
+
+        Returns the HTTP method, HTTP path and data for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            alias (str): The room alias
+            room_id (str): The room to point to
+        """
+
+        query_parameters = {"access_token": access_token}
+        path = ["directory", "room", alias]
+        content = dict()
+        content['room_id'] = room_id
+
+        return ("PUT",
+                Api._build_path(path, query_parameters),
+                Api.to_json(content),)
