@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright © 2018 Damir Jelić <poljar@termina.org.uk>
-# Copyright © 2020 Famedly GmbH
+# Copyright © 2020-2021 Famedly GmbH
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -317,7 +317,7 @@ class Schemas:
             "limited": {"type": "boolean"},
             "prev_batch": {"type": "string"},
         },
-        "required": ["events", "limited", "prev_batch"],
+        "required": ["events", "limited"],
     }
 
     sync = {
@@ -755,7 +755,6 @@ class Schemas:
                     "room_id": {"type": "string", "format": "room_id"},
                     "session_id": {"type": "string"},
                     "session_key": {"type": "string"},
-                    "chain_index": {"type": "integer"},
                 },
                 "required": [
                     "algorithm",
@@ -815,6 +814,17 @@ class Schemas:
                     "creator": {"type": "string", "format": "user_id"},
                     "m.federate": {"type": "boolean", "default": True},
                     "room_version": {"type": "string", "default": "1"},
+                    "predecessor": {
+                          "type": "object",
+                          "properties": {
+                              "event_id": {"type": "string"},
+                              "room_id": {"type": "string", "format": "room_id"}
+                          },
+                          "required": [
+                              "event_id",
+                              "room_id"
+                          ]
+                    }
                 },
                 "required": ["creator"],
             },
@@ -1087,6 +1097,14 @@ class Schemas:
         "required": ["room_id", "servers"],
     }
 
+    room_get_visibility = {
+        "type": "object",
+        "properties": {
+            "room_id": {"type": "string"},
+            "visibility": {"type": "string", "enum": ["private", "public"]},
+        },
+        "required": ["visibility"],
+    }
 
     room_event_id = {
         "type": "object",
@@ -1907,3 +1925,28 @@ class Schemas:
         "user_id": "string",
         "required": ["user_id"],
     }
+
+    room_tombstone = {
+        "type": "object",
+        "properties": {
+            "type": {"type": "string", "const": "m.room.tombstone"},
+            "state_key": {"type": "string", "const": ""},
+            "content": {
+                "type": "object",
+                "properties": {
+                    "body": {"type": "string"},
+                    "replacement_room": {"type": "string", "format": "room_id"},
+                },
+                "required": [
+                    "body",
+                    "replacement_room",
+                ]
+            }
+        },
+        "required": [
+            "sender",
+            "type",
+            "content",
+            "state_key",
+        ],
+    }
